<?php

/**
 * Interface representing a Search API pre- and/or post-query extender.
 *
 * An Extended Processor should make it clear in its description or documentation
 * when it will run and what effect it will have.
 */
interface SearchApiExtendedProcessorInterface {
  /**
   * Construct a n extended processor.
   *
   * @param SearchApiIndex $index
   *   The index on which queries will be generated that will be extended.
   * @param array $options
   *   The extended processor options set for this index.
   */
  public function __construct(SearchApiIndex $index, array $options = array());

  /**
   * Check whether this extended processor is applicable for a certain index.
   *
   * This can be used for hiding the extended processor on the queries configuration
   * form. To avoid confusion, you should only use criteria that are immutable,
   * such as the index's item type. Also, since this is only used for UI
   * purposes, you should not completely rely on this to ensure certain index
   * configurations and at least throw an exception with a descriptive error
   * message if this is violated on runtime.
   *
   * @param SearchApiIndex $index
   *   The index to check for.
   *
   * @return boolean
   *   TRUE if the extended processor can run on the given index; FALSE otherwise.
   */
  public function supportsIndex(SearchApiIndex $index);

  /**
   * Display a form for configuring this extended processor.
   * Since forcing users to specify options for disabled extended processors makes
   * no sense, none of the form elements should have the '#required' attribute
   * set.
   *
   * @return array
   *   A form array for configuring this extended processor, or FALSE if no
   *   configuration is possible.
   */
  public function configurationForm();

  /**
   * Validation callback for the form returned by configurationForm().
   *
   * @param array $form
   *   The form returned by configurationForm().
   * @param array $values
   *   The part of the $form_state['values'] array corresponding to this form.
   * @param array $form_state
   *   The complete form state.
   */
  public function configurationFormValidate(array $form, array &$values, array &$form_state);

  /**
   * Submit callback for the form returned by configurationForm().
   *
   * This method should both return the new options and set them internally.
   *
   * @param array $form
   *   The form returned by configurationForm().
   * @param array $values
   *   The part of the $form_state['values'] array corresponding to this form.
   * @param array $form_state
   *   The complete form state.
   *
   * @return array
   *   The new options array for this callback.
   */
  public function configurationFormSubmit(array $form, array &$values, array &$form_state);

  /**
   * Preprocess a search query.
   *
   * @param SearchApiQuery $query
   *   The object representing the query to be executed.
   */
  public function preprocessSearchQuery(SearchApiQuery $query);

  /**
   * Postprocess search results before display.
   *
   * If a class is used for both pre- and post-processing a search query, the
   * same object will be used for both calls (so preserving some data or state
   * locally is possible).
   *
   * @param array $response
   *   An array containing the search results. See the return value of
   *   SearchApiQueryInterface->execute() for the detailed format.
   * @param SearchApiQuery $query
   *   The object representing the executed query.
   */
  public function postprocessSearchResults(array &$response, SearchApiQuery $query);

}
